#!/bin/bash
#
# Author: Hans-Kristian Bakke
# Date: 14.05.2012
#
####
#
# DESCRIPTION:
# This script searches for .flac files in the directory given as a parameter on script execution and reencodes
# them if their version is older than the FLAC-version currently installed on your system. Alternatively it
# can reencode all files if ALL is set to 1. All subdirectories in the input folder are checked for .flac files
#
# WARNING:
# This script does not preserve cover art embedded in tags through the reencode. Metaflac does support this, so this
# is just the desired behaviour for my usage (I do not want cover art in my files as I store cover art as files).
# All other tags are preserved. Cover art stored as files is not touched in any way.
#
# INTEGRITY:
# The script can safely be interrupted as a backup of the current reencoding file is created in the same directory
# before reencoding. Normally the script will clean up after it self if aborted, but in some extreme cases,
# mainly loosing power or kill -9, the backup have to be recovered manually.
# Look for a file named <filename>.bak in the directory it was working in when interrupted and
# just remove the .bak ending to recover the original FLAC-file.
#
# DEPENDENCIES:
# This script requires that you have FLAC (including metaflac) installed. It is normally enough to
# install flac using your distributions package manager (in Debian you have to add debian-multimedia
# repository to /etc/apt/sources.list first)
#
# USAGE:
# ./flac_reencode.sh <path-to-folder-that-contains-flac-files-to-reencode>
#
####

# Set ALL to 1 if you want to reencode all FLAC-files regardless of version.
# Set to 0 if you only want to reencode FLAC files encoded with previous versions of the FLAC encoder
ALL="0"

# Where to store the log
LOGDIR="/tmp/flac_reencode"
LOGFILE="${LOGDIR}/$(date +'%Y-%m-%d_%H%M%S').log"

####

function ioredirect() {
	exec 3>&1 4>&2
	exec &> >(tee "$LOGFILE")
}

function fail_decode() {
	ERROR=1
	echo "Decoding of $FILE FAILED"
	echo "-> Cleaning up temporary files..."
	rm -f "$WAV"
	rm -f "$TMPTAG"
	echo "-> OK"
}

function fail_encode() {
	ERROR=1
	echo "Encoding of $WAV FAILED"
	echo "-> Cleaning up temporary files..."
	rm -f "$NEWFILE"
	rm -f "$WAV"
	mv "$FILE.bak" "$FILE"
	rm -f "$TMPTAG"
	echo "-> OK"
}

function exit_clean() {
	case $@ in
	2)
		echo "-> Aborted by user!"
		fail_decode
	;;
	3)
		echo "-> Aborted by user!"
		fail_encode
	;;
	esac

	print_errors
	echo -e "\n-> Logfile: $LOGFILE"
	exit $@
}

function print_errors() {
	if [ $ERROR = 1 ]; then
		echo -e "\n-> Error summary:"
		echo "$(grep FAILED "$LOGFILE")"
	else
		echo -e "\n-> No errors"
	fi
}

# Do some checking before running the script
if ! [ -r "$1" ]; then
	echo "$1 is not readable. Ensure the path is valid and readable by your user."
	exit 1
fi
mkdir -p "$LOGDIR" || exit 1
touch "$LOGFILE" || exit 1

ioredirect

# Set some variables
CURRENT_FLAC_VERSION=$(flac -v | awk '{print $2}')
ERROR=0

echo -e "-> Target directory: $1\n"

while read -d $'\0' FILE
do
	if [ $ALL = "0" ]; then
		FILE_FLAC_VERSION=$(metaflac --show-vendor-tag "$FILE" | awk '{print $3}')
		echo "[FLAC v$FILE_FLAC_VERSION] $FILE"
	fi

	if [ $ALL = "1" ] || [ "$FILE_FLAC_VERSION" != "$CURRENT_FLAC_VERSION" ]; then
		NAME=$(echo "$FILE" | sed 's/..\{4\}$//')
		NEWFILE="$NAME.flac"
		WAV="$NAME.wav"

		# DECODE
		trap 'exit_clean 2' SIGINT SIGTERM

		TMPTAG=$(mktemp)
		metaflac --export-tags-to="$TMPTAG" "$FILE"

		if ! flac -s -d "$FILE"; then
			fail_decode
			continue
		fi

		mv "$FILE" "$FILE.bak"
		echo "Decoding $FILE to WAV OK"

		# ENCODE
		trap 'exit_clean 3' SIGINT SIGTERM

		if ! flac -s "$WAV"; then
			fail_encode
			continue
		fi

		rm -f "$WAV"
		rm -f "$FILE.bak"
		echo "Encoding $WAV to FLAC OK"

		metaflac --import-tags-from="$TMPTAG" "$NEWFILE"

		rm -f "$TMPTAG"

		trap - SIGINT SIGTERM
	fi
done < <(find "$1" -type f -iname '*.flac' -print0)

exit_clean 0
